#include <stdio.h>
#include <string.h>

#include "ip.h"
#include "icmp.h"
#include "udp.h"
#include "tcp.h"


DWORD myip; /* local IP address in network byte order */
DWORD netmask; /* network mask */


/* creates IP address as DWORD in network byte order */
DWORD ip_addr(BYTE a1, BYTE a2, BYTE a3, BYTE a4)
{
  return (DWORD)a1|((DWORD)a2<<8)|((DWORD)a3<<16)|((DWORD)a4<<24);
}


/* print given IP address in human readable form to stdout */
void ip_print(DWORD ip)
{
  printf("%d.%d.%d.%d", ((BYTE *)&ip)[0], ((BYTE *)&ip)[1],
    ((BYTE *)&ip)[2], ((BYTE *)&ip)[3]);
}


/* configure local IP address according to given values */
void ip_config(DWORD ip, DWORD netmask)
{
  myip=ip;
  netmask=netmask;
}


/* calculates IP checksum of block of data */
WORD ip_checksum(WORD *wordp, unsigned int bytec)
{
  unsigned long checksum;

  /* clear checksum */
  checksum=0;
  
  /* compute long checksum */
  while (bytec>=2) {
    checksum+=*(wordp++);
    bytec-=2;
  }

  /* odd number of bytes */
  if (bytec) checksum+=(*wordp)&NTOHS(0xff00);

  /* adjust to obtain one's complement checksum */
  while (checksum>0xffff) checksum=(checksum&0xffff)+(checksum>>16);
  
  /* return one's complement of the 16-bit one's complement checksum */
  return ~checksum;
}


/* updates checksum of IP header */
void ip_header_checksum(struct ip_header *iph)
{
  iph->checksum=0;
  iph->checksum=ip_checksum((void *)iph, iph->ihl*4);
}


static struct ip_header iph;

/* input packet feed */
void ip_in(unsigned int len)
{
  unsigned char ihl;
  unsigned int iplen;

  if (len<sizeof(struct ip_header)) return;
  netbuf_read(&iph, sizeof(struct ip_header));

  /*
   * discard packets which belong to other IP address (except for broadcasts),
   * unless our IP address unconfigured
   */
  if (myip!=0 && iph.destip!=myip && (iph.destip|netmask)!=IP_BROADCAST)
    return;

  /* packet reassembly not supported */
  if (iph.fragment&HTONS((IP_FRAG_OFFSET|IP_FRAG_MORE))) return;

  ihl=iph.ihl*4; /* evaluate header length */
  iplen=NTOHS(iph.length);

  /* discard any padding of lower layer protocol and check packet length */
  if (iplen<len) len=iplen;
  if (len<ihl) return;

  len-=ihl; /* evaluate length of the payload and seek to its beginning */
  netbuf_skip(ihl-sizeof(struct ip_header));

  /* dispatch to upper layer */
  switch (iph.protocol) {
    case IP_PROTO_ICMP:
      icmp_in(&iph, len);
      break;
    case IP_PROTO_UDP:
      udp_in(&iph, len);
      break;
    case IP_PROTO_TCP:
      tcp_in(&iph, len);
      break;
  }
}


/* packet ID, incremented for every outgoing IP packet */
static WORD ip_pid;

/* packet transmission */
void ip_out(DWORD destip, BYTE protocol, unsigned int txbuf, unsigned int len)
{
  BYTE *destmac=ETH_BROADCAST;

  /* destip zero? send it as a reply to previously received packet */
  if (destip==0) {
    destip=iph.sourceip;
    destmac=NULL;
  }

  memset(&iph, 0x00, sizeof(struct ip_header));
  iph.ihl=5;
  iph.ver=4;
  iph.ttl=255;
  iph.protocol=protocol;
  iph.identification=ip_pid++;
  iph.sourceip=myip;
  iph.destip=destip;
  len+=sizeof(struct ip_header);
  iph.length=HTONS(len);
  ip_header_checksum(&iph);
  
  netbuf_wr_seek(txbuf+ETH_RESERVED, NB_SEEK_SET);
  netbuf_write(&iph, sizeof(struct ip_header));
  
  /* ARP shall be here */
  eth_out(destmac, HTONS(ETHTYPE_IP), txbuf, len);
}
