#include "platform.h"

#include "leddisp.h"

/* led display and bargraph for PVK40 */

const unsigned char disp_hextab[16]={0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f,0x77,0x7c,0x39,0x5e,0x79,0x71};
const unsigned char disp_selb[5]={0x10, 0x04, 0x02, 0x01 };

/* current display position */
unsigned char disp_phase;

/* exported */
unsigned char disp_data[6];
unsigned int disp_bars;

/* buttons share I/Os with the display */
volatile unsigned char buttons;


/* initialize display I/Os */
void disp_init(void)
{
  TRISD=0x00;
  TRISC&=~DISP_SELC_BARS;
  TRISB&=~DISP_SELB_ALL;
  TRISE&=~0x03;
}


/* show single display position */
void disp_showpos(unsigned char pos)
{
  /* turn off display */
  TRISD=0xff;
  LATB|=DISP_SELB_ALL;

  /* turn off bargraph */
  LATC|=DISP_SELC_BARS;
  LATE|=0x03;

  buttons=~PORTD;
  TRISD=0x00;

  /* bargraph */
  if (pos>=4) {

    /* two upper segmets */
    LATE&=~(disp_bars>>8);

    /* the rest is multiplexed with the display */
    LATD=~(disp_bars&0xff);
    LATC&=~DISP_SELC_BARS; /* turn on bargraph */
    return;
  }

  LATD=disp_data[pos]^0xff;
  LATB&=~disp_selb[pos]; /* turn on display */
}


/* multiplex the display */
void disp_proc(void)
{
  if (++disp_phase>5) disp_phase=0;
  disp_showpos(disp_phase);
}


void disp_hex(unsigned int value)
{
  unsigned char i;

  for (i=4; i--;) {
    disp_data[i]=disp_hextab[value&0x0f];
    value>>=4;
  }
}


void disp_bargraph(unsigned char pos)
{
  disp_bars=((unsigned int)1<<pos)-1;
}
