#include <ctype.h>

#include "nbns.h"
#include "udp.h"


const char *myname="spinet";


/* matches nb name against null term string of ASCII characters */
char nbns_name_match(BYTE *encname, const char *name)
{
  char a;
  unsigned char len;

  len=*(encname++);
  if (len>63) return 0; /* error */

  while (len>=2) {
    a=(*(encname++)-'A')<<4;
    a+=*(encname++)-'A';
    len-=2;
    if (*name=='\0') {
      /* allow trailing spaces and zeros */
      if (a==' ' || a=='\0') continue;
    }
    else if (toupper(a)!=toupper(*(name++))) return 0;
  }

  /* all characters matched? */
  if (*name!='\0') return 0;

  return 1; /* success */
}


UDP_APP_HANDLER(nbns)
{
  unsigned int txbuf;
  struct nbns_header nbnsh;
  struct nbns_name_query_tag qtag;
  unsigned char name[64];
  unsigned char name_len;
  
  if (len<sizeof(struct nbns_header)) return;
  netbuf_read(&nbnsh, sizeof(struct nbns_header));
  
  if (nbnsh.qr==1) return; /* not a query */
  if (nbnsh.opcode!=NBNS_OPCODE_QUERY) return;
  if (nbnsh.qdcount!=HTONS(1)) return; /* only packets with single query supported */

  len-=sizeof(struct nbns_header);

  /* check for name */
  if ((len--)==0) return;
  name_len=name[0]=netbuf_getc();
  if (name_len>63) return; /* probably a pointer, not supported */
  /* there should be name, zero term and query tag */
  if (len<name_len+1+QUERY_TAG_LEN) return;

  /* read the name */
  netbuf_read(name+1,name_len);
  /* there should be just the machine name, no parent domain */
  if (netbuf_getc()!=0) return;
  /* read query tag */
  netbuf_read(&qtag, QUERY_TAG_LEN);

  if (qtag.question_type!=HTONS(QUESTION_TYPE_NB)) return;
  if (qtag.question_class!=HTONS(QUESTION_CLASS_IN)) return;

  if (nbns_name_match(name, myname)) {
    nbnsh.qr=1; /* response */
    nbnsh.aa=1; /* authoritative answer */
    nbnsh.b=0; /* not a broadcast */
    nbnsh.qdcount=HTONS(0);
    nbnsh.ancount=HTONS(1);
    nbnsh.nscount=HTONS(0);
    nbnsh.arcount=HTONS(0);
    qtag.ttl=HTONL(3600);
    qtag.rdlength=HTONS(6);
    qtag.nb_flags=0;
    qtag.nb_addr[0]=myip;

    if (!nic_tx_alloc(&txbuf)) return;
    netbuf_wr_seek(txbuf+UDP_RESERVED, NB_SEEK_SET);
    netbuf_write(&nbnsh, sizeof(struct nbns_header));
    netbuf_write(name,name_len+1);
    netbuf_putc(0); /* zero termination, no parent domain name */
    netbuf_write(&qtag, sizeof(struct nbns_name_query_tag));
    
    len=netbuf_wr_tell()-txbuf-UDP_RESERVED;
    udp_out(0, udph->sourceport, HTONS(137), txbuf, len);
  }
}


void nbns_start(void)
{
  udp_listen(HTONS(137),nbns);
}
