#include <stdlib.h>
#include "udp.h"


struct udp_socket udp_socket_tab[UDP_SOCKETS];


/* assign a socket to a UDP port and register application callback handler */
WORD udp_listen(WORD port, void *handler)
{
  unsigned char i;

  for (i=0; i<UDP_SOCKETS; i++) {
    if (udp_socket_tab[i].port==port) {
      udp_socket_tab[i].app_handler=handler;
      /* port reassigned, caller is responsible to notify
       * previous registered application about this,
       * and prefferably close the port first */
      return port;
    }
  }

  for (i=0; i<UDP_SOCKETS; i++) {
    if (udp_socket_tab[i].port==0) {
      udp_socket_tab[i].port=port;
      udp_socket_tab[i].app_handler=handler;
      return port;
    }
  }

  return 0;
}


/* close given UDP port */
void udp_close_port(WORD port)
{
  unsigned char i;

  for (i=0; i<UDP_SOCKETS; i++) {
    if (udp_socket_tab[i].port==port) {
      udp_socket_tab[i].port=0;
      return;
    }
  }
}


static struct udp_header udph;

/* dispatch UDP datagram to application listening on particular port */
void udp_in(struct ip_header *iph, unsigned int len)
{
  unsigned char i;
  unsigned int udplen;

  if (len<sizeof(struct udp_header)) return;
  netbuf_read(&udph, sizeof(struct udp_header));

  for (i=0; i<UDP_SOCKETS; i++) {
    if (udp_socket_tab[i].port==udph.destport) {
      /* discard any padding of lower layer protocol and check packet length */
      udplen=udph.length;
      if (udplen<len) len=udplen;
      if (len<sizeof(struct udp_header)) return;
      (*(udp_socket_tab[i].app_handler))(iph, &udph, len-sizeof(struct udp_header));
      return;
    }
  }
  
  /* no one listens on such port, datagram discarded */
}


void udp_out(DWORD destip, WORD destport, WORD sourceport, unsigned int txbuf, unsigned int len)
{
  udph.sourceport=sourceport;
  udph.destport=destport;
  len+=sizeof(struct udp_header);
  udph.length=HTONS(len);
  udph.checksum=0;

  netbuf_wr_seek(txbuf+IP_RESERVED, NB_SEEK_SET);
  netbuf_write(&udph, sizeof(struct udp_header));

  ip_out(destip, IP_PROTO_UDP, txbuf, len);
}
